@@ -1,6 +1,6 @@
 from collections.abc import Collection
 from contextlib import AbstractContextManager
-import functools
+import logging
 import os
 from pathlib import Path
 import re
@@ -13,26 +13,30 @@ except ImportError:
     # Try backported to PY<37 `importlib_resources`.
     import importlib_resources as pkg_resources
 
-import serial.tools.list_ports as lp
+from .bluetooth_low_energy_interface import BluetoothLowEnergyInterface
+from .serial_interface import SerialInterface
+from .mirobot_status import MirobotStatus, MirobotAngles, MirobotCartesians
+from .exceptions import ExitOnExceptionStreamHandler, MirobotError, MirobotAlarm, MirobotReset, MirobotAmbiguousPort, MirobotStatusError, MirobotResetFileError, MirobotVariableCommandError
 
-from .serial_device import SerialDevice
-from .mirobot_status import MirobotStatus, MirobotAngleValues, MirobotCartesianValues
-from .exceptions import MirobotError, MirobotAlarm, MirobotReset, MirobotAmbiguousPort, MirobotStatusError, MirobotResetFileError, MirobotVariableCommandError
+os_is_nt = os.name == 'nt'
+os_is_posix = os.name == 'posix'
 
 
 class BaseMirobot(AbstractContextManager):
-    """ A class for managing and maintaining known Mirobot operations. """
+    """ A base class for managing and maintaining known Mirobot operations. """
 
-    def __init__(self, *serial_device_args, debug=False, autoconnect=True, autofindport=True, valve_pwm_values=('65', '40'), pump_pwm_values=('0', '1000'), default_speed=2000, reset_file=None, wait=True, **serial_device_kwargs):
+    def __init__(self, *device_args, debug=False, connection_type='serial', autoconnect=True, autofindport=True, exclusive=True, valve_pwm_values=('65', '40'), pump_pwm_values=('0', '1000'), default_speed=2000, reset_file=None, wait=True, **device_kwargs):
         """
-        Initialization of `BaseMirobot` class.
+        Initialization of the `BaseMirobot` class.
 
         Parameters
         ----------
-        *serial_device_args : List[Any]
-             Arguments that are passed into the `SerialDevice` class.
+        *device_args : Any
+             Arguments that are passed into the `mirobot.serial_device.SerialDevice` or `mirobot.bluetooth_low_energy_interface.BluetoothLowEnergyInterface` class.
         debug : bool
             (Default value = `False`) Whether to print gcode input and output to STDOUT. Stored in `BaseMirobot.debug`.
+        connection_type : str
+            (Default value = `'serial'`) Which type of connection to make to the Mirobot. By default, it will look for a serial port connection (eg. a physical wire connection). For bluetooth, provide `'bluetooth'` or `'bt'` for this parameter. To explicitly specify a serial port connection, use`'serial'` or `'ser'`.
         autoconnect : bool
             (Default value = `True`) Whether to automatically attempt a connection to the Mirobot at the end of class creation. If this is `True`, manually connecting with `BaseMirobot.connect` is unnecessary.
         autofindport : bool
@@ -47,44 +51,63 @@ class BaseMirobot(AbstractContextManager):
             (Default value = `None`) A file-like object, file-path, or str containing reset values for the Mirobot. The default (None) will use the commands in "reset.xml" provided by WLkata to reset the Mirobot. See `BaseMirobot.reset_configuration` for more details.
         wait : bool
             (Default value = `True`) Whether to wait for commands to return a status signifying execution has finished. Turns all move-commands into blocking function calls. Stored `BaseMirobot.wait`.
-        **serial_device_kwargs : Dict
-             Keywords that are passed into the `SerialDevice` class.
+        **device_kwargs : Any
+             Keywords that are passed into the `mirobot.serial_device.SerialDevice` or `mirobot.bluetooth_low_energy_interface.BluetoothLowEnergyInterface` class.
 
         Returns
         -------
         class : `BaseMirobot`
         """
+        self.logger = logging.getLogger(__name__)
+        """ The instance level logger. Of type `logging.Logger` """
+        self.logger.setLevel(logging.DEBUG)
 
+        self.stream_handler = ExitOnExceptionStreamHandler()
+        self.stream_handler.setLevel(logging.DEBUG if debug else logging.INFO)
+
+        formatter = logging.Formatter(f"[Mirobot Init] [%(levelname)s] %(message)s")
+        self.stream_handler.setFormatter(formatter)
+        self.logger.addHandler(self.stream_handler)
+
+        self.device = None
+        """ Object that controls the connection to the Mirobot. Can either be a `mirobot.serial_interface.SerialInterface` or `mirobot.bluetooth_low_energy_interface.BluetoothLowEnergyInterface` class."""
         # Parse inputs into SerialDevice
-        serial_device_init_fn = SerialDevice.__init__
-        args_names = serial_device_init_fn.__code__.co_varnames[:serial_device_init_fn.__code__.co_argcount]
-        args_dict = dict(zip(args_names, serial_device_args))
+        if connection_type.lower() in ('serial', 'ser'):
+            serial_device_init_fn = SerialInterface.__init__
+            args_names = serial_device_init_fn.__code__.co_varnames[:serial_device_init_fn.__code__.co_argcount]
+            args_dict = dict(zip(args_names, device_args))
+            args_dict.update(device_kwargs)
 
-        # check if baudrate was passed in args or kwargs, if not use the default value instead
-        if not ('baudrate' in args_dict or 'baudrate' in serial_device_kwargs):
-            serial_device_kwargs['baudrate'] = 115200
-        # check if stopbits was passed in args or kwargs, if not use the default value instead
-        if not ('stopbits' in args_dict or 'stopbits' in serial_device_kwargs):
-            serial_device_kwargs['stopbits'] = 1
+            args_dict['mirobot'] = self
+            args_dict['exclusive'] = exclusive
+            args_dict['debug'] = debug
+            args_dict['logger'] = self.logger
+            args_dict['autofindport'] = autofindport
 
-        # if portname was not passed in and autofindport is set to true, autosearch for a serial port
-        if autofindport and not ('portname' in args_dict or 'portname' in serial_device_kwargs):
-            self.default_portname = self._find_portname()
-            """ The default portname to use when making connections. To override this on a individual basis, provide portname to each invokation of `BaseMirobot.connect`. """
+            self.device = SerialInterface(**args_dict)
+            self.default_portname = self.device.default_portname
 
-        else:
-            if 'portname' in args_dict:
-                self.default_portname = args_dict['portname']
-            elif 'portname' in serial_device_kwargs:
-                self.default_portname = serial_device_kwargs['portname']
-            else:
-                self.default_portname = None
+        elif connection_type.lower() in ('bluetooth', 'bt'):
+            bluetooth_device_init_fn = BluetoothLowEnergyInterface.__init__
+            args_names = bluetooth_device_init_fn.__code__.co_varnames[:bluetooth_device_init_fn.__code__.co_argcount]
+            args_dict = dict(zip(args_names, device_args))
+            args_dict.update(device_kwargs)
 
-        self.serial_device = SerialDevice(*serial_device_args, **serial_device_kwargs)
+            args_dict['mirobot'] = self
+            args_dict['debug'] = debug
+            args_dict['logger'] = self.logger
+            args_dict['autofindaddress'] = autofindport
+
+            self.device = BluetoothLowEnergyInterface(**args_dict)
+            self.default_portname = self.device.address
+
+        formatter = logging.Formatter(f"[{self.default_portname}] [%(levelname)s] %(message)s")
+        self.stream_handler.setFormatter(formatter)
+        # self.logger.addHandler(self.stream_handler)
 
         self.reset_file = pkg_resources.read_text('mirobot.resources', 'reset.xml') if reset_file is None else reset_file
         """ The reset commands to use when resetting the Mirobot. See `BaseMirobot.reset_configuration` for usage and details. """
-        self.debug = debug
+        self._debug = debug
         """ Boolean that determines if every input and output is to be printed to the screen. """
 
         self.valve_pwm_values = tuple(str(n) for n in valve_pwm_values)
@@ -111,109 +134,43 @@ class BaseMirobot(AbstractContextManager):
         """ Magic method for contextManagers """
         self.disconnect()
 
-    # COMMUNICATION #
+    def __del__(self):
+        """ Magic method for object deletion """
+        self.disconnect()
 
-    def wait_for_ok(self, reset_expected=False, disable_debug=False):
+    def connect(self):
+        self.device.connect()
+
+    def disconnect(self):
+        if getattr(self, 'device', None) is not None:
+            self.device.disconnect()
+
+    @property
+    def is_connected(self):
+        return self.device.is_connected
+
+    @property
+    def debug(self):
+        """ Return the `debug` property of `BaseMirobot` """
+        return self._debug
+
+    @debug.setter
+    def debug(self, value):
         """
-        Continously loops over and collects message output from the serial device.
-        It stops when it encounters an 'ok' or otherwise terminal condition phrase.
+        Set the new value for the `debug` property of `mirobot.base_mirobot.BaseMirobot`. Use as in `BaseMirobot.setDebug(value)`.
+        Use this setter method as it will also update the logging objects of `mirobot.base_mirobot.BaseMirobot` and its `mirobot.serial_device.SerialDevice`. As opposed to setting `mirobot.base_mirobot.BaseMirobot._debug` directly which will not update the loggers.
 
         Parameters
         ----------
-        reset_expected : bool
-            (Default value = `False`) Whether a reset string is expected in the output (Example: on starting up Mirobot, output ends with a `'Using reset pos!'` rather than the traditional `'Ok'`)
-        disable_debug : bool
-            (Default value = `False`) Whether to override the class debug setting. Otherwise one will see status message debug output every 0.1 seconds, thereby cluttering standard output. Used primarily by `BaseMirobot.wait_until_idle`.
+        value : bool
+            The new value for `mirobot.base_mirobot.BaseMirobot._debug`.
 
-        Returns
-        -------
-        output : List[str]
-            A list of output strings upto and including the terminal string.
         """
-        output = ['']
+        self._debug = bool(value)
+        self.stream_handler.setLevel(logging.DEBUG if self._debug else logging.INFO)
+        self.device.setDebug(value)
 
-        ok_eols = ['ok']
-
-        reset_strings = ['Using reset pos!']
-
-        def matches_eol_strings(terms, s):
-            for eol in terms:
-                if s.endswith(eol):
-                    return True
-            return False
-
-        if reset_expected:
-            eols = ok_eols + reset_strings
-        else:
-            eols = ok_eols
-
-        while not matches_eol_strings(eols, output[-1]):
-            msg = self.serial_device.listen_to_device()
-
-            if self.debug and not disable_debug:
-                print(msg)
-
-            if 'error' in msg:
-                raise MirobotError(msg.replace('error: ', ''))
-            if 'ALARM' in msg:
-                raise MirobotAlarm(msg.split('ALARM: ')[1])
-
-            output.append(msg)
-
-            if not reset_expected and matches_eol_strings(reset_strings, msg):
-                raise MirobotReset('Mirobot was unexpectedly reset!')
-                break
-
-        return output[1:]  # don't include the dummy empty string at first index
-
-    def wait_decorator(fn):
-        """
-        A decorator that will use the `wait` argument/keyword for a method to
-        automatically use the `BaseMirobot.wait_for_ok` and conditionally, the `BaseMirobot.wait_until_idle` function calls at the end of the wrapped function.
-
-        Parameters
-        ----------
-        fn : func
-            Function to wrap. Must have the `wait` argument or keyword.
-
-        Returns
-        -------
-        wrapper : func
-            A wrapper that decorates a function.
-        """
-
-        @functools.wraps(fn)
-        def wait_wrapper(self, *args, **kwargs):
-            args_names = fn.__code__.co_varnames[:fn.__code__.co_argcount]
-            args_dict = dict(zip(args_names, args))
-
-            def get_arg(arg_name, default=None):
-                if arg_name in args_dict:
-                    return args_dict.get(arg_name)
-                elif arg_name in kwargs:
-                    return kwargs.get(arg_name)
-                else:
-                    return default
-
-            wait = get_arg('wait', True)
-            disable_debug = get_arg('disable_debug', False)
-            wait_idle = get_arg('wait_idle', True)
-
-            output = fn(self, *args, **kwargs)
-
-            if wait or (wait is None and self.wait):
-                output = self.wait_for_ok(disable_debug=disable_debug)
-
-            if wait_idle:
-                self.wait_until_idle()
-
-            return output
-
-        return wait_wrapper
-
-    # send a message
-    @wait_decorator
-    def send_msg(self, msg, var_command=False, disable_debug=False, wait=None, wait_idle=True):
+    def send_msg(self, msg, var_command=False, disable_debug=False, wait=None, wait_idle=False):
         """
         Send a message to the Mirobot.
 
@@ -224,9 +181,11 @@ class BaseMirobot(AbstractContextManager):
         var_command : bool
             (Default value = `False`) Whether `msg` is a variable command (of form `$num=value`). Will throw an error if does not validate correctly.
         disable_debug : bool
-            (Default value = `False`) Whether to override the class debug setting. Used primarily by `BaseMirobot.wait_until_idle`.
+            (Default value = `False`) Whether to override the class debug setting. Used primarily by ` BaseMirobot.device.wait_until_idle`.
         wait : bool
             (Default value = `None`) Whether to wait for output to end and to return that output. If `None`, use class default `BaseMirobot.wait` instead.
+        wait_idle : bool
+            (Default value = `False`) Whether to wait for Mirobot to be idle before returning.
 
         Returns
         -------
@@ -234,7 +193,7 @@ class BaseMirobot(AbstractContextManager):
             If `wait` is `True`, then return a list of strings which contains message output.
             If `wait` is `False`, then return whether sending the message succeeded.
         """
-        if self.is_connected():
+        if self.is_connected:
             # convert to str from bytes
             if isinstance(msg, bytes):
                 msg = str(msg, 'utf-8')
@@ -244,17 +203,19 @@ class BaseMirobot(AbstractContextManager):
 
             # check if this is supposed to be a variable command and fail if not
             if var_command and not re.fullmatch(r'\$\d+=[\d\.]+', msg):
-                raise MirobotVariableCommandError("Message is not a variable command: " + msg)
+                self.logger.exception(MirobotVariableCommandError("Message is not a variable command: " + msg))
 
             # actually send the message
-            output = self.serial_device.send(msg)
+            output = self.device.send(msg,
+                                      disable_debug=disable_debug,
+                                      wait=(wait or (wait is None and self.wait)),
+                                      wait_idle=wait_idle)
 
-        if self.debug and not disable_debug:
-            print('Message sent: ', msg)
-
-        if (wait is not None and not self.wait) or not wait:
             return output
 
+        else:
+            raise Exception('Mirobot is not Connected!')
+
     def get_status(self, disable_debug=False):
         """
         Get the status of the Mirobot. (Command: `?`)
@@ -262,7 +223,7 @@ class BaseMirobot(AbstractContextManager):
         Parameters
         ----------
         disable_debug : bool
-            (Default value = `False`) Whether to override the class debug setting. Used primarily by `BaseMirobot.wait_until_idle`.
+            (Default value = `False`) Whether to override the class debug setting. Used primarily by `BaseMirobot.device.wait_until_idle`.
 
         Returns
         -------
@@ -271,8 +232,8 @@ class BaseMirobot(AbstractContextManager):
 
         """
         instruction = '?'
-        # we don't want to wait for idling when checking status-- this leads to unbroken recursion!!
-        return self.send_msg(instruction, disable_debug=disable_debug, wait_idle=False)
+        # we don't want to wait for idle when checking status-- this leads to unbroken recursion!!
+        return self.send_msg(instruction, disable_debug=disable_debug, wait=True, wait_idle=False)
 
     def update_status(self, disable_debug=False):
         """
@@ -281,14 +242,26 @@ class BaseMirobot(AbstractContextManager):
         Parameters
         ----------
         disable_debug : bool
-            (Default value = `False`) Whether to override the class debug setting. Used primarily by `BaseMirobot.wait_until_idle`.
+            (Default value = `False`) Whether to override the class debug setting. Used primarily by `BaseMirobot.device.wait_until_idle`.
 
         """
-        status_msg = self.get_status(disable_debug=disable_debug)[0]  # get only the status message and not 'ok'
-        self.status = self._parse_status(status_msg)
+        # get only the status message and not 'ok'
+        status_msg = self.get_status(disable_debug=disable_debug)[0]
+        self._set_status(self._parse_status(status_msg))
 
-    @staticmethod
-    def _parse_status(msg):
+    def _set_status(self, status):
+        """
+        Set the status object given as the instance's new status.
+
+        Parameters
+        ----------
+        status : `mirobot.mirobot_status.MirobotStatus`
+            The new status object of this instance.
+
+        """
+        self.status = status
+
+    def _parse_status(self, msg):
         """
         Parse the status string of the Mirobot and store the various values as class variables.
 
@@ -313,9 +286,9 @@ class BaseMirobot(AbstractContextManager):
             try:
                 state, angles, cartesians, pump_pwm, valve_pwm, motion_mode = regex_match.groups()
 
-                return_angles = MirobotAngleValues(*map(float, angles.split(',')))
+                return_angles = MirobotAngles(**dict(zip('xyzdabc', map(float, angles.split(',')))))
 
-                return_cartesians = MirobotCartesianValues(*map(float, cartesians.split(',')))
+                return_cartesians = MirobotCartesians(*map(float, cartesians.split(',')))
 
                 return_status = MirobotStatus(state,
                                               return_angles,
@@ -325,97 +298,13 @@ class BaseMirobot(AbstractContextManager):
                                               bool(motion_mode))
 
             except Exception as exception:
-                raise Exception([MirobotStatusError(f'Could not parse status message "{msg}"'),
-                                 exception])
+                self.logger.exception(MirobotStatusError(f"Could not parse status message \"{msg}\""),
+                                      exc_info=exception)
             else:
                 return return_status
         else:
-            raise MirobotStatusError(f'Could not parse status message "{msg}"')
+            self.logger.error(MirobotStatusError(f"Could not parse status message \"{msg}\""))
 
-    def wait_until_idle(self, refresh_rate=0.1):
-        """
-        Continuously loops over and refreshes state of the Mirobot.
-        It stops when it encounters an 'Idle' state string.
-
-        Parameters
-        ----------
-        refresh_rate : float
-            (Default value = `0.1`) The rate in seconds to check for the 'Idle' state. Choosing a low number might overwhelm the controller on Mirobot. Be cautious when lowering this parameter.
-
-        Returns
-        -------
-        output : List[str]
-            A list of output strings upto and including the terminal string.
-        """
-        self.update_status(disable_debug=True)
-        if self.status.state == 'Idle':
-            return
-
-        while self.status.state != 'Idle':
-            time.sleep(refresh_rate)
-            self.update_status(disable_debug=True)
-
-    def is_connected(self):
-        """
-        Check if Mirobot is connected.
-
-        Returns
-        -------
-        connected : bool
-            Whether the Mirobot is connected.
-        """
-        return self.serial_device.is_open
-
-    @staticmethod
-    def _find_portname():
-        """
-        Find the port that might potentially be connected to the Mirobot.
-
-        Returns
-        -------
-        device_name : str
-            The name of the device that is (most-likely) connected to the Mirobot.
-        """
-        port_objects = lp.comports()
-
-        if not port_objects:
-            raise MirobotAmbiguousPort("No ports found! Make sure your Mirobot is connected and recognized by your operating system.")
-
-        if len(port_objects) > 1:
-            raise MirobotAmbiguousPort(f"Unable to determine which port to automatically connect to!\nFound these ports: {[p.device for p in port_objects]}.\nTo fix this, please provide port name explicitly.")
-        return port_objects[0].device
-
-    def connect(self, portname=None):
-        """
-        Connect to the Mirobot.
-
-        Parameters
-        ----------
-        portname : str
-            (Default value = `None`) The name of the port to connnect to. If this is `None`, then it will try to use `self.default_portname`. If both are `None`, then an error will be thrown. To avoid this, specify a portname.
-
-        Returns
-        -------
-        ok_msg : List[str]
-            The output from an initial Mirobot connection.
-        """
-        if portname is None:
-            if self.default_portname is not None:
-                portname = self.default_portname
-            else:
-                raise ValueError('Portname must be provided! like so `portname=\'COM3\'`')
-
-        self.serial_device.portname = portname
-
-        self.serial_device.open()
-
-        return self.wait_for_ok(reset_expected=True)
-
-    def disconnect(self):
-        """ Disconnect from the Mirobot. Close the serial device connection. """
-        self.serial_device.close()
-
-    # COMMANDS #
 
     def home_individual(self, wait=None):
         """
@@ -433,7 +322,7 @@ class BaseMirobot(AbstractContextManager):
             If `wait` is `False`, then return whether sending the message succeeded.
         """
         msg = '$HH'
-        return self.send_msg(msg, wait=wait)
+        return self.send_msg(msg, wait=wait, wait_idle=True)
 
     def home_simultaneous(self, wait=None):
         """
@@ -451,7 +340,7 @@ class BaseMirobot(AbstractContextManager):
             If `wait` is `False`, then return whether sending the message succeeded.
         """
         msg = '$H'
-        return self.send_msg(msg, wait=wait)
+        return self.send_msg(msg, wait=wait, wait_idle=True)
 
     def set_hard_limit(self, state, wait=None):
         """
@@ -471,7 +360,7 @@ class BaseMirobot(AbstractContextManager):
             If `wait` is `False`, then return whether sending the message succeeded.
         """
         msg = f'$21={int(state)}'
-        return self.send_msg(msg, wait=wait)
+        return self.send_msg(msg, var_command=True, wait=wait)
 
     # set the soft limit state
     def set_soft_limit(self, state, wait=None):
@@ -492,7 +381,7 @@ class BaseMirobot(AbstractContextManager):
              If `wait` is `False`, then return whether sending the message succeeded.
         """
         msg = f'$20={int(state)}'
-        return self.send_msg(msg, wait=wait)
+        return self.send_msg(msg, var_command=True, wait=wait)
 
     def unlock_shaft(self, wait=None):
         """
@@ -512,23 +401,6 @@ class BaseMirobot(AbstractContextManager):
         msg = 'M50'
         return self.send_msg(msg, wait=wait)
 
-    def go_to_zero(self, wait=None):
-        """
-        Send all axes to their respective zero positions.
-
-        Parameters
-        ----------
-        wait : bool
-            (Default value = `None`) Whether to wait for output to return from the Mirobot before returning from the function. This value determines if the function will block until the operation recieves feedback. If `None`, use class default `BaseMirobot.wait` instead.
-
-        Returns
-        -------
-        msg : List[str] or bool
-            If `wait` is `True`, then return a list of strings which contains message output.
-            If `wait` is `False`, then return whether sending the message succeeded.
-        """
-        return self.go_to_axis(0, 0, 0, 0, 0, 0, 2000, wait=wait)
-
     @staticmethod
     def _generate_args_string(instruction, pairings):
         """
@@ -548,13 +420,13 @@ class BaseMirobot(AbstractContextManager):
             If `wait` is `True`, then return a list of strings which contains message output.
             If `wait` is `False`, then return whether sending the message succeeded.
 message
-             A string containing the base command followed by the correctly formatted arguments.
+            A string containing the base command followed by the correctly formatted arguments.
         """
         args = [f'{arg_key}{value}' for arg_key, value in pairings.items() if value is not None]
 
         return ' '.join([instruction] + args)
 
-    def go_to_axis(self, x=None, y=None, z=None, a=None, b=None, c=None, speed=None, wait=None):
+    def go_to_axis(self, x=None, y=None, z=None, a=None, b=None, c=None, d=None, speed=None, wait=None):
         """
         Send all axes to a specific position in angular coordinates. (Command: `M21 G90`)
 
@@ -572,6 +444,8 @@ message
             (Default value = `None`) Angle of axis 5.
         c : float
             (Default value = `None`) Angle of axis 6.
+        d : float
+            (Default value = `None`) Location of slide rail module.
         speed : int
             (Default value = `None`) The speed in which the Mirobot moves during this operation. (mm/s)
         wait : bool
@@ -589,12 +463,12 @@ message
         if speed:
             speed = int(speed)
 
-        pairings = {'X': x, 'Y': y, 'Z': z, 'A': a, 'B': b, 'C': c, 'F': speed}
+        pairings = {'X': x, 'Y': y, 'Z': z, 'A': a, 'B': b, 'C': c, 'D': d, 'F': speed}
         msg = self._generate_args_string(instruction, pairings)
 
-        return self.send_msg(msg, wait=wait)
+        return self.send_msg(msg, wait=wait, wait_idle=True)
 
-    def increment_axis(self, x=None, y=None, z=None, a=None, b=None, c=None, speed=None, wait=None):
+    def increment_axis(self, x=None, y=None, z=None, a=None, b=None, c=None, d=None, speed=None, wait=None):
         """
         Increment all axes a specified amount in angular coordinates. (Command: `M21 G91`)
 
@@ -612,6 +486,8 @@ message
             (Default value = `None`) Angle of axis 5.
         c : float
             (Default value = `None`) Angle of axis 6.
+        d : float
+            (Default value = `None`) Location of slide rail module.
         speed : int
             (Default value = `None`) The speed in which the Mirobot moves during this operation. (mm/s)
         wait : bool
@@ -630,10 +506,10 @@ message
         if speed:
             speed = int(speed)
 
-        pairings = {'X': x, 'Y': y, 'Z': z, 'A': a, 'B': b, 'C': c, 'F': speed}
+        pairings = {'X': x, 'Y': y, 'Z': z, 'A': a, 'B': b, 'C': c, 'D': d, 'F': speed}
         msg = self._generate_args_string(instruction, pairings)
 
-        return self.send_msg(msg, wait=wait)
+        return self.send_msg(msg, wait=wait, wait_idle=True)
 
     def go_to_cartesian_ptp(self, x=None, y=None, z=None, a=None, b=None, c=None, speed=None, wait=None):
         """
@@ -674,7 +550,7 @@ message
         pairings = {'X': x, 'Y': y, 'Z': z, 'A': a, 'B': b, 'C': c, 'F': speed}
         msg = self._generate_args_string(instruction, pairings)
 
-        return self.send_msg(msg, wait=wait)
+        return self.send_msg(msg, wait=wait, wait_idle=True)
 
     def go_to_cartesian_lin(self, x=None, y=None, z=None, a=None, b=None, c=None, speed=None, wait=None):
         """
@@ -715,7 +591,7 @@ message
         pairings = {'X': x, 'Y': y, 'Z': z, 'A': a, 'B': b, 'C': c, 'F': speed}
         msg = self._generate_args_string(instruction, pairings)
 
-        return self.send_msg(msg, wait=wait)
+        return self.send_msg(msg, wait=wait, wait_idle=True)
 
     def increment_cartesian_ptp(self, x=None, y=None, z=None, a=None, b=None, c=None, speed=None, wait=None):
         """
@@ -756,11 +632,11 @@ message
         pairings = {'X': x, 'Y': y, 'Z': z, 'A': a, 'B': b, 'C': c, 'F': speed}
         msg = self._generate_args_string(instruction, pairings)
 
-        return self.send_msg(msg, wait=wait)
+        return self.send_msg(msg, wait=wait, wait_idle=True)
 
     def increment_cartesian_lin(self, x=None, y=None, z=None, a=None, b=None, c=None, speed=None, wait=None):
         """
-        Linear increment in cartesian coordinates.
+        Linear increment in cartesian coordinates. (Command: `M20 G91 G1`)
 
         Parameters
         ----------
@@ -797,7 +673,7 @@ message
         pairings = {'X': x, 'Y': y, 'Z': z, 'A': a, 'B': b, 'C': c, 'F': speed}
         msg = self._generate_args_string(instruction, pairings)
 
-        return self.send_msg(msg, wait=wait)
+        return self.send_msg(msg, wait=wait, wait_idle=True)
 
     # set the pwm of the air pump
     def set_air_pump(self, pwm, wait=None):
@@ -822,10 +698,10 @@ message
             pwm = self.pump_pwm_values[not pwm]
 
         if str(pwm) not in self.pump_pwm_values:
-            raise ValueError(f'pwm must be one of these values: {self.pump_pwm_values}. Was given {pwm}.')
+            self.logger.exception(ValueError(f'pwm must be one of these values: {self.pump_pwm_values}. Was given {pwm}.'))
 
         msg = f'M3S{pwm}'
-        return self.send_msg(msg, wait=wait)
+        return self.send_msg(msg, wait=wait, wait_idle=True)
 
     def set_valve(self, pwm, wait=None):
         """
@@ -849,10 +725,10 @@ message
             pwm = self.valve_pwm_values[not pwm]
 
         if str(pwm) not in self.valve_pwm_values:
-            raise ValueError(f'pwm must be one of these values: {self.valve_pwm_values}. Was given {pwm}.')
+            self.logger.exception(ValueError(f'pwm must be one of these values: {self.valve_pwm_values}. Was given {pwm}.'))
 
         msg = f'M4E{pwm}'
-        return self.send_msg(msg, wait=wait)
+        return self.send_msg(msg, wait=wait, wait_idle=True)
 
     def start_calibration(self, wait=None):
         """
@@ -924,7 +800,7 @@ message
 
         elif isinstance(reset_file, (str, Path)):
             if not os.path.exists(reset_file):
-                raise MirobotResetFileError("Reset file not found or reachable: {reset_file}")
+                self.logger.exception(MirobotResetFileError("Reset file not found or reachable: {reset_file}"))
             with open(reset_file, 'r') as f:
                 send_each_line(f.readlines())
 
@@ -935,6 +811,6 @@ message
             send_each_line(reset_file.readlines())
 
         else:
-            raise MirobotResetFileError(f"Unable to handle reset file of type: {type(reset_file)}")
+            self.logger.exception(MirobotResetFileError(f"Unable to handle reset file of type: {type(reset_file)}"))
 
         return output
